function [g] = kmo(X);
%  Computes Kaiser-Meyer-Olkin index (KMO)
%
% INPUT
% 
% X : raw data or Pearson correlation matrix
%
% OUTPUT
%
%  g : index value
%
% Written by Urbano Lorenzo-Seva, Rovira i Virgili University (Last update: November 7, 2007)
%

[n,m]=size(X);

%if raw data as input, then compute correlation
if (n~=m), 
    R=corrcoef(X);
else
%Correlation as an input    
    R=X;
end;

%Compute partial correlation

    D  = diag(diag(R^(-1)));
    Ra =  -1 * D^(-1/2) * R^(-1) * D^(-1/2);

%compute KMO       sum(i!=j) r_ij / (sum(i!=j) r_ij + sum(i!=j) ra_ij   )
%r_ij  -> pearson correlation
%ra_ij -> partial correlation

  sumrij = 0;
  sumaij = 0;
  for i=1:m,
	  for j=1:m,
		  if (i~=j),
			  sumrij = sumrij + (R(i,j)  * R(i,j));
			  sumaij = sumaij + (Ra(i,j) * Ra(i,j));
          end;
      end;
  end;
  
  if (sumaij < 0), g = 0.0;
  else g = sumrij /(sumrij + sumaij);
  end;
    
return